﻿/*
 * Created by SharpDevelop.
 * User: pdf
 * Date: 5/2/2017
 * Time: 10:43 AM
 * 
 * V.01 - Initial code
 * 
 * V0.2 - Removed dependency on EasyModbus.DLL by incorporating the code directly
 *      - Removed dependcy on LINQ by changing int16sToFloat concatenation method
 *      - Removed depency on external LED font
 *      - Now targets .NET 2.0 framework, with 5.0 language features
 *      - Catch exception if app is terminated when UCCNC is still running
 * 
 * 
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using EasyModbus;


namespace TCPModbusRemote
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		ModbusServer modbusserver;
		
		public MainForm()
		{
			InitializeComponent();

			modbusserver = new ModbusServer();
			modbusserver.holdingRegistersChanged += new ModbusServer.HoldingRegistersChanged(this.registersChanged);
			modbusserver.Listen();  // port 502 (default)
		}
		
		void MainFormFormClosing(object sender, FormClosingEventArgs e)
		{
			modbusserver.StopListening();
		}
		
		void registersChanged(int registernum, int numchanged)
		{
		
     		// the UCCNC macro must 
			// SetModbusregisters(Startregister, ushort[] Values)
			// where Startregister=0 and the Values array
			// contains values (DRO_COUNT * 2)
			// values must be transmitted in DRO order below

			float fXDRO   = int16sToFloat(modbusserver.holdingRegisters[1],   modbusserver.holdingRegisters[2]);
			float fYDRO   = int16sToFloat(modbusserver.holdingRegisters[3],   modbusserver.holdingRegisters[4]);
			float fZDRO   = int16sToFloat(modbusserver.holdingRegisters[5],   modbusserver.holdingRegisters[6]);
			
			float fFEED   = int16sToFloat(modbusserver.holdingRegisters[7],   modbusserver.holdingRegisters[8]);
			float fSPEED  = int16sToFloat(modbusserver.holdingRegisters[9],   modbusserver.holdingRegisters[10]);
			float fFACT   = int16sToFloat(modbusserver.holdingRegisters[11],  modbusserver.holdingRegisters[12]);
			float fSACT   = int16sToFloat(modbusserver.holdingRegisters[13],  modbusserver.holdingRegisters[14]);
			float fFOVER  = int16sToFloat(modbusserver.holdingRegisters[15],  modbusserver.holdingRegisters[16]);
			float fSOVER  = int16sToFloat(modbusserver.holdingRegisters[17],  modbusserver.holdingRegisters[18]);
			
			float fXDROM  = int16sToFloat(modbusserver.holdingRegisters[19],  modbusserver.holdingRegisters[20]);
			float fYDROM  = int16sToFloat(modbusserver.holdingRegisters[21],  modbusserver.holdingRegisters[22]);
			float fZDROM  = int16sToFloat(modbusserver.holdingRegisters[23],  modbusserver.holdingRegisters[24]);		
			
			// modbusserver runs in a thread
			// we need to join the GUI thread to update controls
			
			try {
				
				MethodInvoker inv = delegate
	    		{
					XDRO.Text      = String.Format("{0:##0.000}", fXDRO);
					YDRO.Text      = String.Format("{0:##0.000}", fYDRO); 
					ZDRO.Text      = String.Format("{0:##0.000}", fZDRO); 
					
					FEED.Text      = String.Format("{0:###.0}",   fFEED); 
					SPEED.Text     = String.Format("{0:####0}",   fSPEED); 
					FACTUAL.Text   = String.Format("{0:###.0}",   fFACT); 
					SACTUAL.Text   = String.Format("{0:####0}",   fSACT); 
					FOVERRIDE.Text = String.Format("{0:###.0}",   fFOVER); 
					SOVERRIDE.Text = String.Format("{0:####0}",   fSOVER);  
					
					XDROM.Text      = String.Format("{0:##0.000}", fXDROM);
					YDROM.Text      = String.Format("{0:##0.000}", fYDROM); 
					ZDROM.Text      = String.Format("{0:##0.000}", fZDROM);
					
	    		};
	 			
				Invoke(inv);
	 			
			} catch (Exception) {}   // ignore the harmless exception if app is terminated while UCCNC is running
		}
		
		float int16sToFloat(short s1, short s2)
		{
		    /* Here we convert the 2 uint16's from consecutive registers
		     * back into a float to display
		    */
		    byte[] s1Bytes = BitConverter.GetBytes(s1);
		    byte[] s2Bytes = BitConverter.GetBytes(s2);
		    
		    // concatenate the byte arrays
		    var s = new MemoryStream();
			s.Write(s2Bytes, 0, s2Bytes.Length);
			s.Write(s1Bytes, 0, s1Bytes.Length);
			var combined = s.ToArray();
			
			// convert array to a float
			return BitConverter.ToSingle(combined, 0);
		}

	}
}
